

proc load-marcu-lisp {file} {
    global RELS GROUP_NODES TEXT_NODES NODE
    reset-all
    load-relations-master $RELS/marcu.rel
    set text [convert-lisp [read-file $file]]
    process-lisp [car $text]

    return 1
}


proc process-lisp {form {par {}}} {
    global NODE TEXT_NODES GROUP_NODES UNKNOWN_RELS RELATIONS
    set role [car $form]

    set id [new-node]
    set text ""
    set rel2par ""
    set Dependents {}
    set NODE($id,role) $role
    set NODE($id,parent) $par

    foreach form1 [cdr $form] {
	set form1 [string trim $form1]
	switch -- [string range $form1 0 3] {
	    "span" {}
	    "prom" {}
	    "text" {
		set NODE($id,text) [string trim [string range $form1 5 end]]
	    }
	    "rel2" {
		set NODE($id,relname) [capitalise [second $form1]]
		
	    }
	    "Nucl" -
	    "Sate" {
		lappend Dependents [process-lisp $form1 $id]
	    }
	    
	    "leaf" {
		set leafid [second $form1]
	    }
	    
	    "sche" {
		set NODE($id,relname)  [capitalise [second $form1]]
		set NODE($par,type) schema
	    }

	    default {
		puts "**UNKNOWN LABEL: /[car $form1]/ ([string range $form1 0 3])"
	    }
	}
    }

    if { $par != {} && [relation-type $NODE($id,relname)] == 0 } {
	# not defined, autodefine
	lappend RELATIONS(rst) $NODE($id,relname)
	
	puts "load-marcu: Unknown relation: $NODE($id,relname) - defining as mononuclear."
	if ![member $NODE($id,relname) $UNKNOWN_RELS] {
	    lappend UNKNOWN_RELS $NODE($id,relname)
	}
    }
    
    if  { $NODE($id,text) != ""} {
	lappend TEXT_NODES $id
	set NODE($id,type) text
	set NODE($id,span) "$id $id"
	regsub -all "\n" $NODE($id,text) " " NODE($id,text)
	regsub -all "  " $NODE($id,text) " " NODE($id,text)
	.segment.editor.text insert end  $text
        new-segment end $id
    } else {
	lappend GROUP_NODES $id
	foreach dep $Dependents {
	    switch -- [relation-type $NODE($dep,relname)] {
		"multinuc" {
		    set NODE($id,type) multinuc
		    break
		}
		"span" {
		    set NODE($id,type) span
		    break
		}

		"schema" {
		    set NODE($id,type) schema
		    break
		}

		default {puts "type: [relation-type $NODE($dep,relname)]"}
	    }
	}

	if {$NODE($id,type) == {}} {
	    puts "Error: unknown type: $id"
	    set NODE($id,type) multinuc
	}
    }	
    
    return $id
}


proc convert-lisp {text} {
    
    set new ""
    for {set c 0} {$c<[string length $text]} {incr c} {
	set char [string index $text $c]
	switch -- $char {
	    "\\" {
		set new "$new[string range $text $c [expr $c + 1]]"
		incr c
	    }

	    "XXXXXX" {
		# was ;
		incr c
		while { [string index $text $c] != "\n"} {incr c}
	    }

	    "#" {
		if {[string index $text [expr $c + 1]] == "|"} {
		    set endpos [string first "|#" $text $c]
		    set c [expr $endpos + 2]
		} else {
		    set new "$new$char"
		}
	    }

	    "\"" {
		set str ""
		set start $c
		while {[string index $text [incr c]] != "\"" || [string index $text [expr $c -1]] == "\\"} {}

		# End of string
		set new "$new[string range $text $start $c]"
		puts "String matched: [string range $text $start $c]"
	    }

	    "\(" {
		set new "$new\{"
	    }

	    "\)" {
		set new "$new\}"
	    }
	    
	    default {
		set new "$new$char"
	    }	    
	}
    }
    return $new
    puts "PARSED: $new"
}


proc tmp {} {
"ROLE SPAN PROM NUC

( Root (span 1 75) (prom 1 4)
   ( Nucleus (span 1 5) (rel2par span) (prom 1 4)
      ( Nucleus (span 1 3) (rel2par Contrast) (prom 1)
         ( Nucleus (leaf 1) (rel2par span) (prom 1) (text  In an age of specialization, the federal judiciary is one of the last bastions of the generalist.) )
         ( Satellite (span 2 3) (rel2par explanation-argumentative) (prom 2)
            ( Nucleus (leaf 2) (rel2par span) (prom 2) (text  A judge must jump from murder to antitrust cases, from arson to securities fraud, ) )
            ( Satellite (leaf 3) (rel2par manner) (prom 3) (text without missing a beat.) )
         )
      )"
}





