######################################
## RSTTool2.0 & Coder3.0
#
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the RSTTool License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/RSTTool/copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.

# Source to load in xml from a file, or from text

proc load-xml {file {encoding {} }} {
    # Loads in the xml into array XMLSEG
    # Also sets LASTSEG and CURRSEG

     #  Parse in the xml form
    set text [read-file $file $encoding]
    load-xml-from-text $text
}

proc load-xml-from-text {text} {
    # We need this so that other routines can extract
    # out between two tags by string search and handle the contained text.

    global LASTSEG XMLSEG CURRSEG
    set segs [split $text {<}]

    set LASTSEG [llength $segs]

    # Show the Progress bar (if source is loaded)
    set Progress 0
    if {[info procs ProgressBar] != {}} {
	set Progress 1
	if [winfo exists .progress] {destroy .progress}
	toplevel .progress
	label .progress.label -text "Loading xml: "
	ProgressBar .progress.bar init 0 $LASTSEG
	pack .progress.label .progress.bar -side left
    } 
    
    if [info exists XMLSEG] {unset XMLSEG}

    for  {set i 1} {$i<$LASTSEG} {incr i} {

	if $Progress {ProgressBar .progress.bar update $i;update}
	set seg [lindex $segs $i]
	
#	puts "\nSEG: $seg"

	set endpos [expr [string first ">" $seg] - 1]

	switch -- [string index $seg $endpos]  {
	    "?" {
		# We have a comment
		set tagstruct "?"
		set XMLSEG($i,endneeded) 0
	    }

	    "/" {
		# This element needs no close tag
		set tagstruct [string range $seg 0 [expr $endpos - 1]]
		set XMLSEG($i,endneeded) 0
	    }

	    default {
		set tagstruct [string range $seg 0 $endpos]
		set XMLSEG($i,endneeded) 1
	    }
	}
	set tagstruct [parse-args $tagstruct]
	set XMLSEG($i,tag)  [lindex $tagstruct 0]
	set XMLSEG($i,args) [lrange $tagstruct 1 end]
	set XMLSEG($i,text) [xml-decode [string range $seg [expr $endpos +2] end]]
#	puts "  TAG: $XMLSEG($i,tag) ARGS: $XMLSEG($i,args) TEXT: $XMLSEG($i,text)"
    }
    set CURRSEG 1
    while { $XMLSEG($CURRSEG,tag) == "?" } {
	incr CURRSEG 
    }
    if $Progress {ProgressBar .progress.bar done;destroy .progress}
}


proc parse-args {args1} {

    set args1 [split-args $args1]
    set len [expr [llength $args1] -1]
    # Start the result with the xml tag
    set result [list [lindex $args1 0]]
    # now add the arguments
    set count 0
    while { $count < $len } {
	set attrib [lindex $args1 [incr count]]
	if { [lindex $args1 [incr count]] != "=" } {
	    puts "Malformed args: $args1 / $count / [lindex $args1 $count]"
	}
	set val [xml-decode [lindex $args1 [incr count]]]
	lappend result [list $attrib $val ]
    }
    return $result
}

proc split-args {string} {
    set parts [split $string {"}]
    # every second element is a quoted value.
    set result {}
    set quote 0
    foreach part $parts {
	if $quote {
	    lappend result $part
	} else {
	    regsub -all "=" $part " = " part
	    regsub -all "\t" $part " " part
	    regsub -all " * " $part " " part
	    set part [string trim $part]
	    set part [split $part " "]
	    
	    set result [concat $result $part]
	}
	
	# toggle quote between 0 and 1
	set quote [expr 1 > $quote]
    }
    return $result
}


proc load-xml-from-text-quick {text} {
    # We need this so that other routines can extract
    # out between two tags by string search and handle the contained text.

    global LASTSEG XMLSEG CURRSEG
    set i 0
    if [info exists XMLSEG] {unset XMLSEG}
    
    set segs [split $text "<"]

    # Show the Progress bar
    if [winfo exists .progress] {destroy .progress}
    toplevel .progress
    label .progress.label -text "Loading xml: "
    ProgressBar .progress.bar init 0 [llength $segs]
    pack .progress.label .progress.bar -side left

    foreach seg $segs {
	incr i
	ProgressBar .progress.bar update $i

	# Test the last char in the <>: < /> -needs no closer; < ?> - a comment
	set endpos [expr [string first ">" $seg] - 1]
	switch -- [string index $seg $endpos]  {
	    "?" {
		# We have a comment
		incr i -1
		continue
	    }

	    "/" {
		# This element needs no close tag
		set tagstruct [string range $seg 0 [expr $endpos - 1]]
		set XMLSEG($i,endneeded) 0
	    }

	    default {
		set tagstruct [string range $seg 0 $endpos]
		set XMLSEG($i,endneeded) 1
	    }
	}
	
	# Locate the tag
	set XMLSEG($i,tag)  [string tolower [stringpop tagstruct]]
	set XMLSEG($i,args) [parse-args  $tagstruct]

	# Find the start of the next segment
	set text1 [string range $seg [expr $endpos +2] end]
	set XMLSEG($i,text) [xml-decode $text1]
    }

    set CURRSEG 1
    set LASTSEG [expr $i + 1]
    
    ProgressBar .progress.bar done
    destroy .progress
}



###################
# Parse-XML


proc parse-xml { {no_closetag_needed {}} {xmldepth 0}} {
    global XMLSEG LASTSEG CURRSEG 
#    puts "CURRSEG: $CURRSEG"
    # interpret the start segment
    set tag $XMLSEG($CURRSEG,tag)
    set lastseg $CURRSEG

    if { $XMLSEG($CURRSEG,endneeded) == 0  || [member $tag $no_closetag_needed] == 1} {
# 	puts "[string repeat " " $xmldepth] -- but no closetag needed"
	return [list $tag $XMLSEG($CURRSEG,args)]
    }

    incr CURRSEG

    # look for end of tagset
    set endtag "/$tag"

    set substruct {}

    while { $CURRSEG < $LASTSEG } {
#	puts "Currseg1: $CURRSEG"
	if { $XMLSEG($CURRSEG,tag) == $endtag } {
# 	    puts "[string repeat " " $xmldepth] -- Matched tag: $endtag"
	    return [list $tag $XMLSEG($lastseg,args) $substruct $XMLSEG($lastseg,text)]
	} else {
	    lappend substruct [parse-xml $no_closetag_needed [expr $xmldepth + 1]]
	}
	incr CURRSEG
    }

    tell-user "Error: parse-xml: Tag $tag of seg $lastseg not matched: $substruct"

    error "Error: Tag $tag not matched: $substruct"
}




proc get-xml-element {form path} {
    return [lindex [get-xml-elements $form $path] 0]
}


proc get-xml-elements {form path} {

    set thistag [lindex $path 0]
    if {$thistag == {} } {
	tell-user "get-xml-elements: null path element"
	break
    }

    # match the formtag and the pathtag
    if { $thistag != [lindex $form 0] } {
	return {}
    }

    # we have a match
    set Tail [lrange $path 1 end]

    if  {$Tail == {} } {
	# this is the element to retrieve
	return [list $form]
    }

    # If only one more tag, allow it to match args
    if { [llength $Tail] == 1} {
	set lasttag [lindex $Tail 0]
	foreach arg [lindex $form 1] {
	    if { $lasttag == [lindex $arg 0] } {
		return [list [lindex $arg 1]]
	    }
	}
    } 

    # else, match on the structure
    set results {}

    foreach subform [lindex $form 2] {
	set result [get-xml-elements $subform $Tail]
	if { $result != {} } {
	    set results [concat $results $result]
	}
    }
    
    return $results
}





proc xml-decode {text} {
    
    regsub -all {&lt;} $text "<" text
    regsub -all {&gt;} $text ">" text
    regsub -all {&amp;} $text {\&} text
    regsub -all {&apos;} $text "'" text
    regsub -all {&quot;} $text {"} text
    regsub -all {&semicolon;} $text {;} text
    return $text
}

# set text [xml-decode "FRED&lt;&gt;&amp;&apos;&quot;END"]

proc xml-encode {text} {

    regsub -all {\&} $text {\&amp;} text
    regsub -all "<" $text {\&lt;} text
    regsub -all ">" $text {\&gt;} text
    regsub -all "'" $text  {\&apos;} text
    regsub -all {"} $text {\&quot;} text
    return $text
}
